package uy.edu.fing.chesstrack.modulovision;

import org.opencv.calib3d.Calib3d;
import org.opencv.core.Mat;
import org.opencv.core.MatOfPoint2f;
import org.opencv.core.Rect;
import org.opencv.core.Size;

import android.util.Log;

public class Calibracion {
	private static final String TAG = "CHESSTRACK::Calibracion";

	private Mat _imagenCalibrada;
	private Mat _imagenOriginal;
	private MatOfPoint2f _vertices;
	private static Rect _rectPOI = null;

	private static Calibracion _instance;

	protected Calibracion() {

	}

	public static Calibracion getInstance() {
		if (_instance == null) {
			_instance = new Calibracion();
		}
		return _instance;
	}

	public boolean calibrar(Mat img) {
		Log.i(TAG, "INI calibrando");
		_imagenOriginal = img;
		_imagenCalibrada = img;
		Size patternSize = new Size(7, 7);
		MatOfPoint2f corners = new MatOfPoint2f();
		boolean ret = Calib3d.findChessboardCorners(_imagenOriginal,patternSize, corners);
		if (ret) {
			Log.i(TAG, "Encontro = " + corners.total());
			//Calib3d.drawChessboardCorners(_imagenCalibrada, patternSize, corners, ret); // only for debug
			_vertices = corners;
		}
		Log.i(TAG, "FIN calibrando = " + ret);
		return ret;
	}

	public Mat getImagenCalibrada() {	// por que no guardar solo el size.. es lo unico qe se necesita
		return _imagenCalibrada;
	}

	public void setImagenCalibrada(Mat imagenCalibrada) {
		this._imagenCalibrada = imagenCalibrada;
	}

	public MatOfPoint2f getVertices() {
		return _vertices;
	}

	public void setVertices(MatOfPoint2f vertices) {
		this._vertices = vertices;
	}

	public static Rect getRectPOI() {
		return _rectPOI;
	}

	public static void setRectPOI(Rect rectPOI) {
		Calibracion._rectPOI = rectPOI;
	}

}
